# -*- coding: utf-8 -*-
"""
Created on Sun Oct  4 15:42:46 2020

@author: 44752
"""

# add in about spaces for them all

def atbash_decrypt(cipher_text):
    
    """ 
    This is a function which decrypts atbash 
    The function takes an argument of the cipher text and 
    returns the plain text
    """
    
    # ensures that the cipher_text is a string
    if type(cipher_text) != str:
        raise TypeError("cipher_text must be type string")
    
    # list which defines the alphabet
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","J","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z"]
    
    # empty list which will be used to store the plain text
    decrypt = []                              
    
    # for loops through all letter of cipher and decrypts 
    # checks that it is a capital A-Z
    # gets the position in the alphabet A-Z - 0-25
    # gets corresponding letter using atbash
    for i in range(len(cipher_text)):   
        
        letter = cipher_text[i]
        
        # checks that each element is a capital letter
        if letter not in alphabet and letter != " ":
            raise TypeError("letter must be capital between A-Z", letter)
        
        # if isn't a space decrypt
        if letter != " ":
            alpha_position = alphabet.index(letter) 
            new_position = (-alpha_position  % 25) + 1        
            new_letter = alphabet[new_position - 1]   
            decrypt.append(new_letter)            
        else:
            decrypt.append(" ")
            
    # plain text to be outputted
    plain_text = "".join(decrypt)
    
    return(plain_text)
    
def atbash_encrypt(plain_text):
    
    """ 
    This is a function which encrypts atbash 
    The function takes an argument of the plain text and 
    returns the cipher text
    """
    
    # ensures that the cipher_text is a string
    if type(plain_text) != str:
        raise TypeError("plain_text must be type string")
    
    # list which defines the alphabet
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","J","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z"]
    
    # empty list which will be used to store the plain text
    decrypt = []                              
    
    # for loops through all letter of cipher and decrypts 
    # checks that it is a capital A-Z
    # gets the position in the alphabet A-Z - 0-25
    # gets corresponding letter using atbash
    for i in range(len(plain_text)):   
        
        letter = plain_text[i]
        
        # checks that each element is a capital letter
        if letter not in alphabet and letter != " ":
            raise TypeError("letter must be capital between A-Z", letter)
        
        # if isn't a space decrypt
        if letter != " ":
            alpha_position = alphabet.index(letter) 
            new_position = (-alpha_position  % 25) + 1        
            new_letter = alphabet[new_position - 1]   
            decrypt.append(new_letter)            
        else:
            decrypt.append(" ")
            
    # plain text to be outputted
    cipher_text = "".join(decrypt)
    
    return(cipher_text)


def ceaser_decrpyt(cipher_text, shift):
    
    """
    This is a function which decrypts a Ceaser cipher
    Takes arguments of the cipher text and the shift wanting to be applied
    """
    
    # ensures cipher_text is a string type
    if type(cipher_text) != str:
        raise TypeError("cipher_text must be type string")
    
    # ensures shift is an integer
    if type(shift) != int:
        raise TypeError("shift must be type int")
        
    # list which defines the alphabet
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","J","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z"]
    
    # empty list which will be used to store the plain text
    decrypt = []                                   
    
    # for loops through all letter of cipher and decrypts 
    # checks that it is a capital A-Z
    # gets the position in the alphabet A-Z - 0-25
    # gets corresponding letter using atbash
    for i in range(len(cipher_text)): 
           
        letter = cipher_text[i]
        
         # checks that each element is a capital letter
        if letter not in alphabet and letter != " ":
            raise TypeError("letter must be capital between A-Z", letter)
        
        # if it isn't a space decryprt
        if letter != " ":
            alpha_position = alphabet.index(letter)       
            
            # shifting the letters with ceaser
            if alpha_position - shift >  0 :                    
                new_letter = alphabet[alpha_position - shift]
                decrypt.append(new_letter)        
            else: 
                new_letter = alphabet[alpha_position % 25- shift]      
                decrypt.append(new_letter)  
        else:
            decrypt.append(" ")
        
        # add to plain text
        plain_text = "".join(decrypt)
    
    return(plain_text)

def ceaser_encrpyt(plain_text, shift):
    
    """
    This is a function which encrypts a Ceaser cipher
    Takes arguments of the plain and the shift wanting to be applied
    """
    
    # ensures cipher_text is a string type
    if type(plain_text) != str:
        raise TypeError("plain_text must be type string")
    
    # ensures shift is an integer
    if type(shift) != int:
        raise TypeError("shift must be type int")
        
    shift = 26 - shift
        
    # list which defines the alphabet
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","J","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z"]
    
    # empty list which will be used to store the plain text
    decrypt = []                                   
    
    # for loops through all letter of cipher and decrypts 
    # checks that it is a capital A-Z
    # gets the position in the alphabet A-Z - 0-25
    # gets corresponding letter using atbash
    for i in range(len(plain_text)): 
           
        letter = plain_text[i]
        
         # checks that each element is a capital letter
        if letter not in alphabet and letter != " ":
            raise TypeError("letter must be capital between A-Z", letter)
        
        # if it isn't a space decryprt
        if letter != " ":
            alpha_position = alphabet.index(letter)       
            
            # shifting the letters with ceaser
            if alpha_position - shift >  0 :                    
                new_letter = alphabet[alpha_position - shift]
                decrypt.append(new_letter)        
            else: 
                new_letter = alphabet[alpha_position % 25- shift]      
                decrypt.append(new_letter)  
        else:
            decrypt.append(" ")
        
        # add to cipher text
        cipher_text = "".join(decrypt)
    
    return(cipher_text)


def scytale_decrypt(cipher_text, band):
    
    """
    This is a function which decrypts a scytale cipher
    Takes arguments of the cipher_text and band
    Band is how many letters in the row of your scytale
    """
      # ensures cipher_text is a string type
    if type(cipher_text) != str:
        raise TypeError("cipher_text must be type string")
    
    # ensures band is an integer 
    if type(band) != int:
        raise TypeError("band must be type int")
    
    # list of letters and dictionary
    list_of_text =  list(cipher_text.strip())
    d = {}
    start = 0

    
   # adding _'s into the list that are extra that will go at the end of plain text
    if len(list_of_text) % band != 0:
        val_0 = int(len(list_of_text)/band) + 1
        which_col = (len(list_of_text) % band)
       
        for num in range(which_col+1, band+1):
            insert = num * val_0-1
            list_of_text.insert(insert, '_')
            
    val = int(len(list_of_text)/band)
    
    # codes the list into a dictionary 
    for x in range(0, val):
        if len(list_of_text[start::val]) == band:  
            d["row{0}".format(x)] = list_of_text[start::val]
            start = start + 1
        else:
            break
    
    # transforming dictionary to readable letters
    dic_list = list(d.values())
    joint_list = [j for i in dic_list for j in i]
    joint_list [:] = (value for value in joint_list  if value != "_")
    plain_text = "".join(joint_list)
        
    return(plain_text)


def scytale_encrypt(plain_text, band):
   
    """
    This is a function which encryps a scytale cipher
    Takes arguments of the plain_text and band
    Band is how many letters in the row of your scytale
    """
    
     # ensures plain_text is a string type
    if type(plain_text) != str:
        raise TypeError("plain_text must be type string")
    
    # ensures band is an integer 
    if type(band) != int:
        raise TypeError("band must be type int")
        
        
    # list of letters and dictionary
    list_of_text =  list(plain_text.strip())
    d = {}
    val = int(len(list_of_text)/band)
    
    
    # adding "_" to the end
    if len(list_of_text) % band != 0:
        remainder = band - len(list_of_text) % band
        list_of_ = ["_"] * remainder
        list_of_text = list_of_text + list_of_

    
    # to iterate through in different chunks
    start = 0
    end = band 
    
    # asigning each row to the dictionary
    for x in range(0, val+1):
        d["row{0}".format(x)] = list_of_text[start:end]
        start = start + band 
        end = end + band 
    
    # converts dict to list and creates new cipher_text list
    dic_list = list(d.values())
    cipher_text = []
    
    # append values from dict to create cipher text
    for j in range(band):
        for i in range(val+1):
            cipher_text.append(dic_list[i][j])
            
    # get rid of "_" and convert list to string
    cipher_text[:] = (value for value in cipher_text if value != "_")
    cipher_text = "".join(cipher_text)
    
    return(cipher_text)


# next is vigenre decrypt

def vigenre_decrpyt(cipher_text, keyword):
    
    """
    This is a function which decrypts a vigenere cipher
    Takes arguments of the cipher_text and keyword
    """
    
    # ensures cipher_text is a string type
    if type(cipher_text) != str:
        raise TypeError("cipher_text must be type string")
    
    # ensures keyword is an integer 
    if type(keyword) != str:
        raise TypeError("keyword must be type string")
        
    # list which defines the alphabet
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","J","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z"]
    
    def order_alpha(letter):
    
        """
        This is a function which takes a letter in the alphabet
        Then creates a list starting with this letter, wrapping round the 
        alphabet
        """
        pos = alphabet.index(letter)
    
        new_alpha =  alphabet[pos:26] + alphabet[0:pos] 
    
        return(new_alpha)
    
    # this defines my col and rows for the vigenere decoder    
    col_dict = {}
    row_dict = {}
    plain_text = []
    for x in alphabet: 
        col_dict["col{0}".format(x)] = order_alpha(x)
        row_dict["row{0}".format(x)] = order_alpha(x)
      
    # makes cipher text and keyword into a list
    list_of_text =  list(cipher_text.strip())
    list_of_keyword =  list(keyword.strip())
    keyword_iterator = 0
    cipher_text_iterator = 0
    
    # so going through each letter, looking at the corresponding keyword
    # then by analysing the vigenere square, outputting the correct letter
    for cipher_text_iterator in range(len(list_of_text)):
        
        if keyword_iterator == len(list_of_keyword):
            keyword_iterator = 0
            
            # error checking to check it is one of the alphabet
            if list_of_keyword[keyword_iterator] not in alphabet:
                raise TypeError("keyword must be a capital letter")
            else:
                row = row_dict["row"+list_of_keyword[keyword_iterator]]
                keyword_iterator = keyword_iterator + 1
        else:
            if list_of_keyword[keyword_iterator] not in alphabet:
                raise TypeError("keyword must be a capital letter")
            else:
                row = row_dict["row"+list_of_keyword[keyword_iterator]]
                keyword_iterator = keyword_iterator + 1
        
        if list_of_text[cipher_text_iterator] not in alphabet:
            raise TypeError("cipher text must be a capital letter")
        else:
            which_col = row.index(list_of_text[cipher_text_iterator])
        col_name = alphabet[which_col]
        plain_text.append(col_name)
    
    # join all the letters together
    plain_text = "".join(plain_text)
    
    return(plain_text)
    
    
def vigenre_encrpyt(plain_text, keyword):
    
    """
    This is a function which encrypts using a vigenere cipher
    Takes arguments of the plain_text and keyword
    """
    
    # ensures cipher_text is a string type
    if type(plain_text) != str:
        raise TypeError("cipher_text must be type string")
    
    # ensures keyword is an integer 
    if type(keyword) != str:
        raise TypeError("keyword must be type string")
        
    # list which defines the alphabet
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","J","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z"]
    
    def order_alpha(letter):
    
        """
        This is a function which takes a letter in the alphabet
        Then creates a list starting with this letter, wrapping round the 
        alphabet
        """
        pos = alphabet.index(letter)
    
        new_alpha =  alphabet[pos:26] + alphabet[0:pos] 
    
        return(new_alpha)
    
    # this defines my col and rows for the vigenere decoder    
    col_dict = {}
    row_dict = {}
    cipher_text = []
    for x in alphabet: 
        col_dict["col{0}".format(x)] = order_alpha(x)
        row_dict["row{0}".format(x)] = order_alpha(x)
      
    # makes plain text and keyword into a list 
    # with iterator integers to iterate through text / keyword
    list_of_text =  list(plain_text.strip())
    list_of_keyword =  list(keyword.strip())
    keyword_iterator = 0
    plain_text_iterator = 0
    
    # so going through each letter, looking at the corresponding keyword
    # then by analysing the vigenere square, outputting the correct letter
    # checks if the keyword needs to be wrapped round again
    for plain_text_iterator in range(len(list_of_text)):
        
        if keyword_iterator == len(list_of_keyword):
            
            keyword_iterator = 0
            
            # error checking to check it is one of the alphabet
            if list_of_keyword[keyword_iterator] not in alphabet:
                raise TypeError("keyword must be a capital letter")
            else:
                col = alphabet.index(keyword[keyword_iterator])
                keyword_iterator = keyword_iterator + 1
        else:
            if list_of_keyword[keyword_iterator] not in alphabet:
                raise TypeError("keyword must be a capital letter")
            else:
                col = alphabet.index(keyword[keyword_iterator])
                keyword_iterator = keyword_iterator + 1
        
        if list_of_text[plain_text_iterator] not in alphabet:
            raise TypeError("plain text must be a capital letter")
        else:
            plain_text_letter = list_of_text[plain_text_iterator]
            row = row_dict["row"+plain_text_letter]
            cipher_text_letter = row[col]
            cipher_text.append(cipher_text_letter )
    
    #join all the letters together
    cipher_text = "".join(cipher_text)
    
    return(cipher_text)


def playfair_encrypt(plain_text, keyword):

    """
    This is a function which encrypts using a playfair cipher
    Takes arguments of the plain_text and keyword
    """
    # ensures cipher_text is a string type
    if type(plain_text) != str:
        raise TypeError("cipher_text must be type string")
    
    # ensures keyword is an integer 
    if type(keyword) != str:
        raise TypeError("keyword must be type string")
        
    # list which defines the alphabet, no J in playfair
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z"]
     
    # list of letters in keyword and plain text
    keyword_list_strip = list(keyword.strip())
    plain_text_list = list(plain_text.strip())
    
    # get rid of duplicates in keyword
    keyword_list = []
    [keyword_list.append(x) for x in keyword_list_strip if x not in keyword_list] 
    
    # setting up the playfair grid
    for letter in keyword:
        if letter in alphabet:
            alphabet.remove(letter)
    
    playfair_grid = keyword_list + alphabet
    
    # adding in X to plaintext when duplicates occur
    plain_text_list_length = len(plain_text_list)
    
    for i in range(plain_text_list_length):
        if i == 0:
            pass
        else:
            if plain_text_list[i] == plain_text_list[i-1]:
                plain_text_list.insert(i, "X")
    
    # add X at the end
    plain_text_list_length = len(plain_text_list)
    
    if plain_text_list_length % 2 !=0:
        plain_text_list.insert(plain_text_list_length, "X")
    
    # define row/cols
    row1 = [0,1,2,3,4]
    row2 = [5,6,7,8,9]
    row3 = [10,11,12,13,14]
    row4 = [15,16,17,18,19]
    row5 = [20,21,22,23,24]
    
    row1_grid = playfair_grid[0:5]
    row2_grid = playfair_grid[5:10]
    row3_grid = playfair_grid[10:15]
    row4_grid = playfair_grid[15:20]
    row5_grid = playfair_grid[20:25]
    
    print(row1_grid,row2_grid,row3_grid,row4_grid)
    
    col1 = [0,1,2,3,4]
    col2 = [5,6,7,8,9]
    col3 = [10,11,12,13,14]
    col4 = [15,16,17,18,19]
    col5 = [20,21,22,23,24]
    
    # split lists into pair
    list_pairs = []
    i = 0
    j = 1
    
    # so for all letters within my cipher text
    for x in range(len(plain_text_list)):                           
        if j < len(plain_text_list):                                
            list_pairs.append(plain_text_list[i] + plain_text_list[j]) 
            i = i + 2                                           
            j = j + 2
    
    # decide if letter in same row/col and which one
    for pair in list_pairs:
        
        first_letter = pair[0]
        second_letter = pair[1]
        
        playfair_pos_1 = playfair_grid.index(first_letter)
        playfair_pos_2 = playfair_grid.index(second_letter)
        
        #print(first_letter, playfair_pos_1,second_letter, playfair_pos_2)
        
        if playfair_pos_1 in row1 and playfair_pos_2 in row1:
            cipher_first_letter = row1_grid[playfair_pos_1 - 1]
            cipher_second_letter = row1_grid[playfair_pos_2 - 1]
      
            print("false")
        
    
    return(cipher_first_letter, cipher_second_letter)

text5 = "MUKOFLRGXLIUIHALCIBICTEIVNMZFDLCFLRLRYTPOMSYLWGFCLPTLWLOMLFRLOLBODOPUQPOPOLELBTPKOEIHUFPNGMOLXKCRLPNNQEQFRYRZUTSLUIVOQOMQLMLLFRYQYILMLZICIIQLFOFOCIFDFNHZLCFIDTHOKTKUNOSHGUGZGTPBLGCWZGMIFTNGKTPTCMNKOXDXLRNKTLBGUEXNTYRPOFRRLCLXBTKIFESHOPTNGILQFIAEHFDXLKTRGEUTGQZILRLPOFELCBREHCDERTGUYILMLZUTPLPIZOHXBPLNQYRIHCIEFSTZURNLOFEUQPOENMULEHNIAXLFQLNAMIFRFKCXLGURZPOFLLEUYOQOMEPLZLAIVILMLALSYNQRGGIGKQNOFRLENMULOBDLXXNGHZLPTVILULBZRFLEFBICODYKZMQILRKCOXBTKIFVILUIVLXGRQLCVCBGURZPOGRHTQYOQLPLUPHTLYIDKXEHARHGRFEPODLBVXLRYQNHONGGMPECTLUHTCIFLELVILUIVKTYRADZEFRRGEDVNNQVIHNGCRVRBCTKOELLBKMXCQLFEMGTLUGEPHUVCFEOMQOOINQOMIFVILNBLDXLTGCEXNTYRPODCRVRBSQCTZTBNNQGKELCLZRFLIQOFGIYITMTPZRLBMLLCFCGHLFTKEHMKUYOWCTBUKTUYILMLVLBLUGMTVRCTFLROZKIQKQRGGQOMPZLXVKHAFLSGLTILPNKOIUKTYRQORCRXUORFGQHLZGPNLCFLRGXLEGTLAPLBNHXDSQCTZTGSLBGKELLBMTVRGCXLGQHGGCUMOHNYPSHOEAOMTCARLXUQPOGZILKAOMOFLOEOHQKOGUNHOFTPTCODOPMLLOQKEZBFTPHWGLESFRHNAQLHHSKQLBGQTZLBRLOFPONGHGTPLOMKZY"
test = "ABBAOAIRHEGOIERNGONREOIHREGOIWHREGIORHGIOHHHSGHAODIHGOIR"
steph = "MEETMEATTREFFORESTSTATION"
len(steph)
z = playfair_encrypt(steph, "GLAMORGAN")
w = playfair_encrypt("NC", "GLAMORGAN")
print("hello")



# need to do the shifting across of cols
# then need to work out what to do with no rows/cols


























alphabet_dictionary={"A":0,"B":1,"C":2,"D":3,"E":4,"F":5,"G":6,"H":7,"I":8,
                     "J":9,"K":10,"L":11,"M":12,"N":13,"O":14,"P":15, "Q":16,
                     "R":17,"S":18,"T":19,"U":20 ,"V":21,"W":22,"X":23,"Y":24,
                     "Z":25,"a":0,"b":1,"c":2,"d":3,"e":4,"f":5,"g":6,"h":7,
                     "i":8,"j":9,"k":10,"l":11,"m":12,"n":13,"o":14,"p":15,
                     "q":16,"r":17,"s":18,"t":19,"u":20 ,"v":21,"w":22,"x":23,
                     "y":24,"z":25," ":" "}

alphabet={"A":0,"B":1,"C":2,"D":3,"E":4,"F":5,"G":6,"H":7,"I":8,
           "J":9,"K":10,"L":11,"M":12,"N":13,"O":14,"P":15, "Q":16,
           "R":17,"S":18,"T":19,"U":20 ,"V":21,"W":22,"X":23,"Y":24,
           "Z":25, " ":" "}

alphabet_list=["A","B","C","D","E","F","G","H","I",
           "J","K","L","M","N","O","P", "Q",
           "R","S","T","U" ,"V","W","X","Y",
           "Z"]

alphabet_list_playfair=["A","B","C","D","E","F","G","H","I",
           "K","L","M","N","O","P", "Q",
           "R","S","T","U" ,"V","W","X","Y",
           "Z"]

def type_str_error(letters:str):
    """[Raises type error if letters isn't a string]
    Args:
        letters (str): [letter being passed in]

    Raises:
        TypeError: [if letters isn't a string, raise error]
    """
    if type(letters) != str:
        raise TypeError(f"{letters} must be type string")

        
def type_int_error(shift:int):
    """[Raises type error if letters isn't a string]
    Args:
        letters (str): [letter being passed in]

    Raises:
        TypeError: [if letters isn't a string, raise error]
    """
    if type(shift) != int:
        raise TypeError(f"{shift} must be an int")


def type_alphabet_error(letters:str):
    """
    """
    alphabet = ["A","B","C","D","E","F","G","H",
                "I","J","K","L","M","N","O","P",
                "Q","R","S","T","U" ,"V","W","X",
                "Y","Z","a","b","c","d","e","f",
                "g","h","i","j","k","l","m","n",
                "o","p","q","r","s","t","u" ,"v",
                "w","x","y","z"]
    for letter in letters:
        if letter not in alphabet and letter != " ":
                raise TypeError("""Letter in CipherText must be 
                                letter between A-Z""", letter)
                            
def type_duplicate_letter_error(letters:str):
    
    if letters != "":
        if len(letters) != len(set(letters)):
            
            raise TypeError("Duplicates in permutation")
                                
    
def get_pos_alphabet(letters:str)->list:
    """gets the numerical position of letters in the alphabet,
        e.g a=0,b=1,...,z=25
    Args:
        letters (str): letters passed through

    Returns:
        list: numerical equivelent of letter
    """
    alpha_positions=[alphabet_dictionary[letter] for letter in letters]

    return alpha_positions 

def get_atbashed_pos_alphabet(numbers):
    """gets the atbashed position equivelent of letter

    Args:
        numbers ([int]): positions of the letters from the cipher text

    Returns:
        new_position: the decrypted atbash position
    """
    new_position = [(25-alpha_position) %26 for alpha_position in numbers if alpha_position!=" "] 
    
    return new_position

def get_atbashed_pos_alphabet_encrypt(numbers):
    """gets the atbashed position equivelent of letter

    Args:
        numbers ([int]): positions of the letters from the cipher text

    Returns:
        new_position: the decrypted atbash position
    """
    new_position = [-(alpha_position  +1) %26  for alpha_position in numbers if alpha_position!=" "] 
    
    return new_position

def get_key(val):
    """gets the letter from a specificed postion. e,g.
       1=a,2=b,...,z=25

    Args:
        val ([int]): alphabet position number

    Returns:
        [type]: corresponding alphabet number
    """
    for key, value in alphabet.items():
        
        if value == val:
            return key
    return "key doesn't exist"

def get_alphabet_from_pos(letter_pos):
    """applies get_key to all letters to get the alphabet from
       it's position number
    Args:
        letter_pos ([int]): alphabet position number
    Returns:
        all alphabet leters from it's poisiton number,
        this shoud be the final plain text
    """
    alpha=[get_key(letter) for letter in letter_pos]
    
    return alpha

def get_ceaser_pos(letters_pos,shift):
    """ges the ceaser decrypted position of letter

    Args:
        letters_pos ([list of ints]): list of ints which are 
        letter positions
        shift ([int]): [the value for your ceaser cipher shifted]

    Returns:
        [list]: [the ceaser decypted value with the shift applied to it]
    """

    shifted=[(letter-shift) % 26 for letter in letters_pos if letter != " "]

    return shifted

def get_ceaser_pos_decrypt(letters_pos,shift):
    """ges the ceaser encrypted position of letter

    Args:
        letters_pos ([list of ints]): list of ints which are 
        letter positions
        shift ([int]): [the value for your ceaser cipher shifted]

    Returns:
        [list]: [the ceaser encrypted value with the shift applied to it]
    """


    shifted=[(letter+shift) % 26 for letter in letters_pos if letter != " "]

    return shifted

def order_alpha(letter:str)->list:
    """
    will create a list with the letter passed through first,
    then will go through all rest of the alphabet and loops back
    E.g order_alpha("D") = ["D","E","F",...,"Z","A","B","C"]

    Parameters
    ----------
    letter : str
        DESCRIPTION.

    Returns
    -------
    list
        DESCRIPTIONs

    """
    pos = alphabet_list.index(letter)

    new_alpha =  alphabet_list[pos:26] + alphabet_list[0:pos] 

    return(new_alpha)

def vigenere_permutation(permutation):
    """creates the vigenre square top row with a permutation

    Args:
        permutation str: the permutation used to create
        the vigenre square. Can't be a duplicate

    Returns:
        list: the vigenre square top row in the form of a list
    """
    
    permutation=[char for char in permutation]
    not_in_perma=[letter for letter in alphabet_list if letter not in permutation]
    order_alpha_perm=permutation+not_in_perma
    
    return order_alpha_perm
    

def create_viginere_table(permutation):
    """Creates viginere table based on if there is a permutation ornot

    Args:
        permutation str: the permutation used to create
        the vigenre square. Can't be a duplicate. If permutation=""
        then normal A,B,C square. 

    Returns:
        table dictionary: All rows of the viginere list
    """
    
    table={}
    
    if permutation == "":
        table={letter:order_alpha(letter) for letter in alphabet_list}
    else:
        top_row=vigenere_permutation(permutation)
        for letter in alphabet_list:
            table[letter]=top_row
            top_row=top_row[1:] + top_row[:1]
        
    
    return table

def keyword_text_match_up(text, keyword):
    """repeats keyword so that it matches the text e.g
    keyword=fam, text="ABCDEFG" it will output
    keyword_repeat=["F","A","M,"F","A","M","F] &
    text_list=["A","B","C,"D","E","F","G]
    Args:
        text str: the cipher or plain text to do encryption/decryption on
        keyword str: The keyword used for encryption/decryption
    Returns:
        keyword_repeat,text_list [lists]: The text with it's keyword
        repeated so it matches the text
    """
    text_list=[char for char in text]
    text_length=len(text_list)
    
    keyword_list=[char for char in keyword]
    keyword_length=len(keyword)
    
    number_repeats=int(text_length/keyword_length)
    leftover=text_length%keyword_length
    
    keyword_repeat=keyword_list*number_repeats+keyword_list[0:leftover]
    
    return keyword_repeat,text_list
    
def viginere_get_table_decrypt(table,keyword_letter,text_letter):
    """outputs the decrypted letter based on the dictionary table,
       keyword letter and cipher text letter wanting to decrypt    

    Args:
        table ([dictionary]): dictionary of the table 
        keyword_letter ([str]): [letter from keyword]
        text_letter ([str]): [letter wanting to decnrypt]

    Returns:
        [str]: [decrypted letter]
    """
    
    row=table[keyword_letter]
    where_in_row=row.index(text_letter)
    out_letter=alphabet_list[where_in_row]
    
    return out_letter


def viginere_get_table_encrypt(table,keyword_letter,text_letter):
    """outputs the encrypted letter based on the dictionary table,
       keyword letter and plain text letter wanting to encrypt    

    Args:
        table ([dictionary]): dictionary of the table 
        keyword_letter ([str]): [letter from keyword]
        text_letter ([str]): [letter wanting to ecnrypt]

    Returns:
        [str]: [encrypted letter]
    """

    row=table[keyword_letter]
    col=alphabet_list.index(text_letter)
    out_letter=row[col]

    return out_letter

def playfair_grid_list(keyword):
    """produces the playfair grid list based on the keyword
    Args:
        keyword ([str]): [keyword used to produce playfair table]
    Returns:
        [list]: [the playfair grid as a lit]
    """
    keyword_list_strip = list(keyword.strip())
    key_word_no_dups = list(dict.fromkeys(keyword_list_strip))
    other_letter=[letter for letter in alphabet_list_playfair if letter not in key_word_no_dups]
    playfair_grid = key_word_no_dups + other_letter
    
    return playfair_grid
    
def playfair_deal_with_dups(text):
    """playfair can't deal with conseuctive letters being duplicated,
    this adds in an X between conseuctive duplciated letters

    Args:
        text ([str]): plain text to get rid of dups

    Returns:
        [message]: list of text with the X's added where there are conseuctive duplicates
    """
    message = list(text.strip())
    for i in range(1,len(message)):
       
        if message[i] == message[i-1]:
                message.insert(i, "X")

    return message

def get_play_fair_row_col(playfair_grid,letter):
    """[get which row, col a letter is within a playfair grid]

    Args:
        playfair_grid ([list]): [playfair grid]
        letter ([str]): [letter you wan the row/column for]

    Returns:
        [row, col]: [the row and column for the letter passed in
        using the playfair grid passed in]
    """
    
    pos_in_grid=playfair_grid.index(letter)
    row,col=divmod(pos_in_grid,5)
    
    return row, col

def playfair_put_into_pairs(text):
    """takes a list of strings and puts them in to pairs,
        adds extra X on the end incase where len text is odd 

    Args:
        text ([list]): [lsit of strings e.g ["A","B","C","D"]
        

    Returns:
        [list]: [that will go to ["AB","CD]]
    """
    if len(text) % 2 !=0:
        text.append("X")
    
    split = [first + second for first,second in zip(text[::2], text[1::2])]

    return split

def playfair_decrypt_row_pair(grid, pair):
    """if two letters are in the same row,
    this is the method to decrypt 

    Args:
        grid ([list]): [list of the playfair grid]
        pair ([str]): [pair of strings in form e.g "AB"]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns decrypted letters as a tuple]
    """

    pos_in_list_1=grid.index(pair[0])
    pos_in_list_2=grid.index(pair[1])

    if pos_in_list_1 % 5 !=0:
        first_decrpyted_letter=grid[(pos_in_list_1-1)]
    else:
        first_decrpyted_letter=grid[(pos_in_list_1+4)]
    if pos_in_list_2 % 5 !=0:
        second_decrpyted_letter=grid[(pos_in_list_2-1)]
    else:
        second_decrpyted_letter=grid[(pos_in_list_2+4)]

    return first_decrpyted_letter,second_decrpyted_letter

def playfair_decrypt_col_pair(grid, pair):
    """if two letters are in the same col,
    this is the method to decrypt 

    Args:
        grid ([list]): [list of the playfair grid]
        pair ([str]): [pair of strings in form e.g "AB"]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns decrypted letters as a tuple]
    """

    pos_in_list_1=grid.index(pair[0])
    pos_in_list_2=grid.index(pair[1])

    if pos_in_list_1 >4:
        first_decrpyted_letter=grid[(pos_in_list_1-5)]
    else:
        first_decrpyted_letter=grid[(pos_in_list_1+20)]
    if pos_in_list_2 >4:
        second_decrpyted_letter=grid[(pos_in_list_2-5)]
    else:
        second_decrpyted_letter=grid[(pos_in_list_2+20)]

    return first_decrpyted_letter,second_decrpyted_letter


def playfair_decrypt_rectangle(grid, x_1,y_1,x_2,y_2):
    """if two letters are in not in the same col/row,
    this is the method to decrypt 

    Args:
        grid ([list]): [playfair grid]
        x_1 ([): [first letter, row]]
        y_1 ([first letter col]): [first letter, col]]
        x_2 ([second ]): [second letter, row]]
        y_2 ([type]): [second letter, col]]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns decrypted letters as a tuple]
    """

    first_decrpyted_letter=grid[(x_1*5+y_2)]
    second_decrpyted_letter=grid[(x_2*5+y_1)]

    return first_decrpyted_letter,second_decrpyted_letter


def playfair_decrypt_pair(playfair_list, pair):
    """Takes the playfair grid and a pair, calculates which method
    of decryption it should use (row, col, rectangle, then proceeds
    to do this decryption)

    Args:
        playfair_list ([list]): [playfair grid]
        pair ([str]): [pair you want to decrypt i.e "AB"]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns decrypted letters as a tuple]
    """
    (l1_row,l1_col)=get_play_fair_row_col(playfair_grid=playfair_list,
                                                    letter=pair[0])

    (l2_row,l2_col)=get_play_fair_row_col(playfair_grid=playfair_list,
                                                    letter=pair[1])
    if l1_row==l2_row:
        (first_decrpyted_letter, 
        second_decrpyted_letter)=playfair_decrypt_row_pair(grid=playfair_list, pair=pair)
    elif l1_col==l2_col:
        (first_decrpyted_letter, 
        second_decrpyted_letter)=playfair_decrypt_col_pair(grid=playfair_list, pair=pair)
    else:
        (first_decrpyted_letter, 
        second_decrpyted_letter)=playfair_decrypt_rectangle(grid=playfair_list, 
                                                            x_1=l1_row,
                                                            y_1=l1_col,
                                                            x_2=l2_row,
                                                            y_2=l2_col)

    return first_decrpyted_letter, second_decrpyted_letter

def playfair_encrypt_row_pair(grid, pair):
    """if two letters are in the same row,
    this is the method to decrypt 

    Args:
        grid ([list]): [list of the playfair grid]
        pair ([str]): [pair of strings in form e.g "AB"]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns encrypted letters as a tuple]
    """

    pos_in_list_1=grid.index(pair[0])
    pos_in_list_2=grid.index(pair[1])

    end_of_row=[4,9,14,19,24]

    if (pos_in_list_1 not in end_of_row) | (pos_in_list_1==0):
        first_encrypted_letter=grid[(pos_in_list_1+1)]
    else:
        first_encrypted_letter=grid[(pos_in_list_1-4)]
    if (pos_in_list_2 not in end_of_row) | (pos_in_list_2==0):
        second_encrypted_letter=grid[(pos_in_list_2+1)]
    else:
        second_encrypted_letter=grid[(pos_in_list_2-4)]

    return first_encrypted_letter,second_encrypted_letter

def playfair_encrypt_col_pair(grid, pair):
    """if two letters are in the same col,
    this is the method to decrypt 

    Args:
        grid ([list]): [list of the playfair grid]
        pair ([str]): [pair of strings in form e.g "AB"]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns encrypted letters as a tuple]
    """

    pos_in_list_1=grid.index(pair[0])
    pos_in_list_2=grid.index(pair[1])

    if pos_in_list_1 >19:
        first_encrypted_letter=grid[(pos_in_list_1-20)]
    else:
        first_encrypted_letter=grid[(pos_in_list_1+5)]
    if pos_in_list_2 >19:
        second_encrypted_letter=grid[(pos_in_list_2-20)]
    else:
        second_encrypted_letter=grid[(pos_in_list_2+5)]

    return first_encrypted_letter,second_encrypted_letter


def playfair_encrypt_rectangle(grid, x_1,y_1,x_2,y_2):
    """if two letters are in not in the same col/row,
    this is the method to decrypt 

    Args:
        grid ([list]): [playfair grid]
        x_1 ([): [first letter, row]]
        y_1 ([first letter col]): [first letter, col]]
        x_2 ([second ]): [second letter, row]]
        y_2 ([type]): [second letter, col]]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns encrypted letters as a tuple]
    """

    first_encrypted_letter=grid[(x_1*5+y_2)]
    second_encrypted_letter=grid[(x_2*5+y_1)]

    return first_encrypted_letter,second_encrypted_letter


def playfair_encrypt_pair(playfair_list, pair):
    """Takes the playfair grid and a pair, calculates which method
    of encryption it should use (row, col, rectangle, then proceeds
    to do this decryption)

    Args:
        playfair_list ([list]): [playfair grid]
        pair ([str]): [pair you want to encrypt i.e "AB"]

    Returns:
        [first_decrpyted_letter,
        second_decrpyted_letter]: [Returns encrypted letters as a tuple]
    """
    (l1_row,l1_col)=get_play_fair_row_col(playfair_grid=playfair_list,
                                                    letter=pair[0])

    (l2_row,l2_col)=get_play_fair_row_col(playfair_grid=playfair_list,
                                                    letter=pair[1])
    if l1_row==l2_row:
        (first_encrypted_letter, 
        second_encrypted_letter)=playfair_encrypt_row_pair(grid=playfair_list, pair=pair)
    elif l1_col==l2_col:
        (first_encrypted_letter, 
        second_encrypted_letter)=playfair_encrypt_col_pair(grid=playfair_list, pair=pair)
    else:
        (first_encrypted_letter, 
        second_encrypted_letter)=playfair_encrypt_rectangle(grid=playfair_list, 
                                                            x_1=l1_row,
                                                            y_1=l1_col,
                                                            x_2=l2_row,
                                                            y_2=l2_col)

    return first_encrypted_letter, second_encrypted_letter


def unpack_list_of_tuples(list_tuples):
    """takes the list of tuples e.g
    [("A","B),("C","D)] and unpacks them into a normal list
    like ["A","B","C","D]

    Args:
        list_tuples ([list]): [description]

    Returns:
        [text]: [list] the untupled list
    """

    text=[]
    for i in range(len(list_tuples)):
        first,second=list_tuples[i]
        text.append(first)
        text.append(second)

    return text


from tools import (type_str_error, 
                   type_alphabet_error,
                   type_int_error,
                   type_duplicate_letter_error,
                   get_pos_alphabet,
                   get_atbashed_pos_alphabet,
                   get_alphabet_from_pos,
                   get_atbashed_pos_alphabet_encrypt,
                   get_ceaser_pos,
                   get_ceaser_pos_decrypt,
                   keyword_text_match_up,
                   create_viginere_table,
                   viginere_get_table_decrypt,
                   viginere_get_table_encrypt,
                   playfair_grid_list,
                   playfair_deal_with_dups,
                   playfair_put_into_pairs,
                   playfair_decrypt_pair,
                   playfair_encrypt_pair,
                   unpack_list_of_tuples)

def atbash_decrypt(cipher_text):
    """Decrypts cipher text into plain text
       using the atbash ciper

    Args:
        cipher_text ([string]): [string of cipher texts to decrypt]

    Returns:
        [plain_text_joined]: [the decryped cipher text i.e plain text]
    """
    # error catching
    type_str_error(cipher_text)
    type_alphabet_error(cipher_text)
    
    # transformation letters to atbashed equivalent
    letter_pos=get_pos_alphabet(cipher_text)
    atbashed_pos=get_atbashed_pos_alphabet(letter_pos)
    plain_text=get_alphabet_from_pos(atbashed_pos)
    
    # joins plain text into a string
    plain_text_joined = "".join(plain_text)
    
    
    return plain_text_joined

def atbash_encrypt(plain_text):
    """Encrypts plain text into cipher text
       using the atbash ciper

    Args:
        plain_text ([string]): [string of plain text to encryptt]

    Returns:
        [cipher_text_joined]: [the decryped cipher text i.e plain text]
    """
    # error catching
    type_str_error(plain_text)
    type_alphabet_error(plain_text)
    
    # transformation letters to atbashed equivalent
    letter_pos=get_pos_alphabet(plain_text)
    atbashed_pos=get_atbashed_pos_alphabet_encrypt(letter_pos)
    cipher_text=get_alphabet_from_pos(atbashed_pos)
    
    # joins plain text into a string
    cipher_text_joined = "".join(cipher_text)
    
    
    return cipher_text_joined


def ceaser_decrypt(cipher_text, shift):
    """Decrypts cipher text into plain text
       using the ceaser ciper

    Args:
        cipher_text ([string]): [string of cipher texts to decrypt]
        shift ([int]): [The shift key applied to ceaser]

    Returns:
        [plain_text_joined]: [the decrypted cipher text i.e plain text]
    """
    # error catching
    type_str_error(cipher_text)
    type_alphabet_error(cipher_text)
    type_int_error(shift)

    letter_pos=get_pos_alphabet(cipher_text)
    ceaser_pos=get_ceaser_pos(letters_pos=letter_pos,
                              shift=shift)

    plain_text=get_alphabet_from_pos(ceaser_pos)
    
    # joins plain text into a string
    plain_text_joined = "".join(plain_text)
    
    
    return plain_text_joined


def ceaser_encrypt(plain_text, shift):
    """ENcrypts plain text into cipher text
       using the ceaser ciper

    Args:
        plain_text ([string]): [string of plain text to encryptt]
        shift ([int]): [The shift key applied to ceaser]

    Returns:
        [cipher_text_joined]: [the decryped cipher text i.e plain text]
    """

    # error catching
    type_str_error(plain_text)
    type_alphabet_error(plain_text)
    type_int_error(shift)

    letter_pos=get_pos_alphabet(plain_text)
    ceaser_pos=get_ceaser_pos_decrypt(letters_pos=letter_pos,
                                      shift=shift)

    cipher_text=get_alphabet_from_pos(ceaser_pos)
    
    # joins plain text into a string
    cipher_text_joined = "".join(cipher_text)
    
    
    return cipher_text_joined

def vigenere_decrypt(cipher_text, keyword, permutation=""):
    """Decrypts cipher text into plain text
       using the viginere ciper

    Args:
        cipher_text ([string]): [string of cipher texts to decrypt]
        keyword ([str]): [The keyword used to decrypt the cipher]
        permutation([str]) [The permutation of the vigenre table,
        defaults to standard "ABC"]
    Returns:
        [plain_text_joined]: [the decrypted cipher text i.e plain text]
    """
    # error catching
    type_str_error(cipher_text)
    type_alphabet_error(cipher_text)
    type_str_error(keyword)
    type_alphabet_error(keyword)
    type_str_error(permutation)
    type_duplicate_letter_error(permutation)
    
    # capitalise 
    keyword=keyword.upper()
    cipher_text=cipher_text.upper()
    permutation=permutation.upper()

    # remove white spaces
    keyword=keyword.replace(" ", "")
    cipher_text=cipher_text.replace(" ", "")
    permutation=permutation.replace(" ","")


    # create table
    table=create_viginere_table(permutation)


    # list of keyword next to cipher text
    key,cipher=keyword_text_match_up(text=cipher_text, 
                                     keyword=keyword) 
    
    # decrypt using table, keyword list & cipher
    plain_text=[viginere_get_table_decrypt(table,k,text) for [k,text] in zip(key,cipher)]    
    # joins plain text into a string
    plain_text_joined = "".join(plain_text)
    
    
    return plain_text_joined

def vigenere_encrypt(plain_text, keyword, permutation=""):
    """ENcrypts plain text into cipher text
       using the viginere ciper

    Args:
        plain_text ([string]): [string of plain text to encryptt]
        keyword ([str]): [The keyword used to encrypt the cipher]
        permutation([str]) [The permutation of the vigenre table,
        defaults to standard "ABC"]

    Returns:
        [cipher_text_joined]: [the decryped cipher text i.e plain text]
    """
    # error catching
    type_str_error(plain_text)
    type_alphabet_error(plain_text)
    type_str_error(keyword)
    type_alphabet_error(keyword)
    type_str_error(permutation)
    type_duplicate_letter_error(permutation)

    # capitalise 
    keyword=keyword.upper()
    plain_text=plain_text.upper()
    permutation=permutation.upper()

    # remove white spaces
    keyword=keyword.replace(" ", "")
    plain_text=plain_text.replace(" ", "")
    permutation=permutation.replace(" ","")

      # create table
    table=create_viginere_table(permutation)
    
    # list of keyword next to plain text
    key,text=keyword_text_match_up(text=plain_text, 
                                     keyword=keyword) 
    
    # decrypt using table, keyword list & cipher
    cipher_text=[viginere_get_table_encrypt(table,k,text) for [k,text] in zip(key,text) ]    
    # joins plain text into a string
    cipher_text_joined = "".join(cipher_text)
    
    
    return cipher_text_joined
    
def playfair_decrypt(cipher_text,keyword):
    """Decrypts cipher text into plain text
       using the playfair ciper

    Args:
        cipher_text ([string]): [string of cipher texts to decrypt]
        keyword ([str]): [The keyword used to decrypt the cipher]
    Returns:
        [plain_text_joined]: [the decrypted cipher text i.e plain text]
    """
    # catching errros
    type_str_error(cipher_text)
    type_alphabet_error(cipher_text)
    type_str_error(keyword)
    type_alphabet_error(keyword)

        # capitalise 
    keyword=keyword.upper()
    cipher_text=cipher_text.upper()

    # remove white spaces
    keyword=keyword.replace(" ", "")
    cipher_text=cipher_text.replace(" ", "")
    cipher_text=cipher_text.replace("J", "I")


    grid_list=playfair_grid_list(keyword)
    plain_text_adjusted_dups=playfair_deal_with_dups(cipher_text)

    paired_list=playfair_put_into_pairs(plain_text_adjusted_dups)
    descrypted_tuples=[playfair_decrypt_pair(playfair_list=grid_list,pair=pairs) for pairs in paired_list]

    plain_text=unpack_list_of_tuples(list_tuples=descrypted_tuples)
    plain_text_joined = "".join(plain_text)
    plain_text_no_x = plain_text_joined.replace("X","")

    return plain_text_no_x

def playfair_encrypt(plain_text,keyword):
    """ENcrypts plain text into cipher text
       using the playfair ciper

    Args:
        plain_text ([string]): [string of plain text to encryptt]
        keyword ([str]): [The keyword used to encrypt the cipher]

    Returns:
        [chiper_text_no_x]: [the decrytped cipher text i.e plain text]
    """
    # catching errors
    type_str_error(plain_text)
    type_alphabet_error(plain_text)
    type_str_error(keyword)
    type_alphabet_error(keyword)

    # capitalise 
    keyword=keyword.upper()
    plain_text=plain_text.upper()

    # remove white spaces
    keyword=keyword.replace(" ", "")
    plain_text=plain_text.replace(" ", "")
    plain_text=plain_text.replace("J", "I")

    grid_list=playfair_grid_list(keyword)
    plain_text_adjusted_dups=playfair_deal_with_dups(plain_text)

    paired_list=playfair_put_into_pairs(plain_text_adjusted_dups)
    descrypted_tuples=[playfair_encrypt_pair(playfair_list=grid_list,pair=pairs) for pairs in paired_list]

    chiper_text=unpack_list_of_tuples(list_tuples=descrypted_tuples)
    chiper_text_joined = "".join(chiper_text)
    chiper_text_no_x = chiper_text_joined.replace("X","")

    return chiper_text_no_x

# folder structure
# .innit. files
# read me
# license 
# CI/CD files



import unittest

from tools import (type_str_error,
                   type_alphabet_error,
                   type_int_error,
                   type_duplicate_letter_error,
                   get_pos_alphabet,
                   get_atbashed_pos_alphabet,
                   get_alphabet_from_pos,
                   get_ceaser_pos,
                   get_ceaser_pos_decrypt,
                   order_alpha,
                   vigenere_permutation,
                   create_viginere_table,
                   keyword_text_match_up,
                   viginere_get_table_decrypt,
                   viginere_get_table_encrypt,
                   playfair_grid_list,
                   playfair_deal_with_dups,
                   get_play_fair_row_col,
                   playfair_put_into_pairs,
                   playfair_decrypt_pair,
                   playfair_encrypt_row_pair,
                   playfair_decrypt_row_pair,
                   playfair_decrypt_col_pair,
                   playfair_encrypt_col_pair,
                   playfair_decrypt_rectangle,
                   playfair_encrypt_rectangle,
                   unpack_list_of_tuples)

class test_tools(unittest.TestCase):

    def test_type_str_error(self):
        
        self.assertRaises(TypeError, type_str_error, 2)
        self.assertRaises(TypeError, type_str_error, ["A"])
        self.assertRaises(TypeError, type_str_error, ("A", "B", "C"))
        
    def test_type_alphabet_error(self):
        
        self.assertRaises(TypeError, type_alphabet_error, "3")
        self.assertRaises(TypeError, type_alphabet_error, ["!?@"])
    
    def test_type_duplicate_letter_error(self):
        
        self.assertRaises(TypeError, type_duplicate_letter_error, "AA")
        
        
    def test_get_pos_alphabet(self):
        
        sample=["A","V", "F"]
        
        expected=[0, 21, 5]
        
        result=get_pos_alphabet(sample)
        
        self.assertListEqual(expected, result)
        
    def test_get_atbash_decrypt_pos_alphabet(self):
        
        sample=[0, 21, 5]
        
        expected=[25,4,20]
        
        result=get_atbashed_pos_alphabet(sample)
        
        self.assertListEqual(expected, result)
    
    def test_get_alphabet_from_pos(self):
        
        sample=[25,4,20]
        
        expected=["Z", "E", "U"]
        
        result=get_alphabet_from_pos(sample)
        
        self.assertListEqual(expected, result)

    def test_type_int_error(self):

        self.assertRaises(TypeError, type_int_error, "2")
        self.assertRaises(TypeError, type_int_error, ["A"])
        self.assertRaises(TypeError, type_int_error, ("A", "B", "C"))

    def test_get_ceaser_pos(self):

        sample=[4,25,14,11,9]
    

        result=get_ceaser_pos(letters_pos=sample,
                        shift=11)

        expected=[19,14,3,0,24]

        self.assertListEqual(result, expected)

    def test_get_ceaser_pos_decrypt(self):

        
        sample=[19,14,3,0,24]
    

        result=get_ceaser_pos_decrypt(letters_pos=sample,
                        shift=11)

       
        expected=[4,25,14,11,9]

        self.assertListEqual(result, expected)

    def test_order_alpha(self):
        
        input_var="D"
        
        result=order_alpha(input_var)
        
        expected=['D','E','F','G','H','I','J','K','L','M','N','O','P',
                  'Q','R','S','T','U','V','W','X','Y','Z','A','B','C']

        self.assertListEqual(result, expected)
        
    def test_vigenere_permutation(self):
        
        input_var="SWAN"
        
        result=vigenere_permutation("SWAN")
        
        expected=['S','W','A','N','B','C','D','E','F','G','H','I','J',
                  'K','L','M','O','P','Q','R','T','U','V','X','Y','Z']
        
        self.assertListEqual(result, expected)        
        
    def test_create_viginere_table(self):
        
        input_var="THISLEPGYWNOMARKDBFCJQUVXZ"
        
        result=create_viginere_table(input_var)
        
        expected={'A': ['T','H','I','S','L','E','P','G','Y','W','N','O','M',
                        'A','R','K','D','B','F','C','J','Q','U','V','X','Z'],
                  'B': ['H','I','S','L','E','P','G','Y','W','N','O','M','A',
                        'R','K','D','B','F','C','J','Q','U','V','X','Z','T'],
                  'C': ['I','S','L','E','P','G','Y','W','N','O','M','A','R',
                        'K','D','B','F','C','J','Q','U','V','X','Z','T','H'],
                  'D': ['S','L','E','P','G','Y','W','N','O','M','A','R','K',
                        'D','B','F','C','J','Q','U','V','X','Z','T','H','I'],
                  'E': ['L','E','P','G','Y','W','N','O','M','A','R','K','D',
                        'B','F','C','J','Q','U','V','X','Z','T','H','I','S'],
                  'F': ['E','P','G','Y','W','N','O','M','A','R','K','D','B',
                        'F','C','J','Q','U','V','X','Z','T','H','I','S','L'],
                  'G': ['P','G','Y','W','N','O','M','A','R','K','D','B','F',
                        'C','J','Q','U','V','X','Z','T','H','I','S','L','E'],
                  'H': ['G','Y','W','N','O','M','A','R','K','D','B','F','C',
                        'J','Q','U','V','X','Z','T','H','I','S','L','E','P'],
                  'I': ['Y','W','N','O','M','A','R','K','D','B','F','C','J',
                        'Q','U','V','X','Z','T','H','I','S','L','E','P','G'],
                  'J': ['W','N','O','M','A','R','K','D','B','F','C','J','Q',
                        'U','V','X','Z','T','H','I','S','L','E','P','G','Y'],
                  'K': ['N', 'O','M','A','R','K','D','B','F','C','J','Q','U',
                        'V','X','Z','T','H','I','S','L','E','P','G','Y','W'],
                  'L': ['O','M','A','R','K','D','B','F','C','J','Q','U','V',
                        'X','Z','T','H','I','S','L','E','P','G','Y','W','N'],
                  'M': ['M','A','R','K','D','B','F','C','J','Q','U','V','X',
                        'Z','T','H','I','S','L','E','P','G','Y','W','N','O'],
                  'N': ['A','R','K','D','B','F','C','J','Q','U','V','X','Z',
                        'T','H','I','S','L','E','P','G','Y','W','N','O','M'],
                  'O': ['R','K','D','B','F','C','J','Q','U','V','X','Z','T',
                        'H','I','S','L','E','P','G','Y','W','N','O','M','A'],
                  'P': ['K','D','B','F','C','J','Q','U','V','X','Z','T','H',
                        'I','S','L','E','P','G','Y','W','N','O','M','A','R'],
                  'Q': ['D','B','F','C','J','Q','U','V','X','Z','T','H','I',
                        'S','L','E','P','G','Y','W','N','O','M','A','R','K'],
                  'R': ['B','F','C','J','Q','U','V','X','Z','T','H','I','S',
                        'L','E','P','G','Y','W','N','O','M','A','R','K','D'],
                  'S': ['F','C','J','Q','U','V','X','Z','T','H','I','S','L',
                        'E','P','G','Y','W','N','O','M','A','R','K','D','B'],
                  'T': ['C','J','Q','U','V','X','Z','T','H','I','S','L','E',
                        'P','G','Y','W','N','O','M','A','R','K','D','B','F'],
                  'U': ['J','Q','U','V','X','Z','T','H','I','S','L','E','P',
                        'G','Y','W','N','O','M','A','R','K','D','B','F','C'],
                  'V': ['Q','U','V','X','Z','T','H','I','S','L','E','P','G',
                        'Y','W','N','O','M','A','R','K','D','B','F','C','J'],
                  'W': ['U','V','X','Z','T','H','I','S','L','E','P','G','Y',
                        'W','N','O','M','A','R','K','D','B','F','C','J','Q'],
                  'X': ['V','X','Z','T','H','I','S','L','E','P','G','Y','W',
                        'N','O','M','A','R','K','D','B','F','C','J','Q','U'],
                  'Y': ['X','Z','T','H','I','S','L','E','P','G','Y','W','N',
                        'O','M','A','R','K','D','B','F','C','J','Q','U','V'],
                  'Z': ['Z','T','H','I','S','L','E','P','G','Y','W','N','O',
                        'M','A','R','K','D','B','F','C','J','Q','U','V','X']}   
        
        self.maxDiff=None
        self.assertDictEqual(expected, result)
        
    def test_keyword_text_match_up(self):
        
        result1,result2=keyword_text_match_up(text="FGQVEQYONMCCHAXTPBAC", 
                              keyword="FAMILY")      
    
        expected1=['F','A','M','I','L','Y','F','A','M','I','L','Y',
                  'F','A','M','I','L','Y','F','A']
        
        expected2=['F','G','Q','V','E','Q','Y','O','N','M','C',
                   'C','H','A','X','T','P','B','A','C']
        
        self.assertEqual(result1, expected1)
        self.assertEqual(result2, expected2)
    
    def test_viginere_get_table_decrypt(self):
        
        vigin_table=create_viginere_table("") 
        result=viginere_get_table_decrypt(table=vigin_table,
                                          keyword_letter="M",
                                          text_letter="Q")
        
        expected="E"
        
        self.assertEqual(result, expected)
        
    def test_viginere_get_table_encrypt(self):
        
        vigin_table=create_viginere_table("") 
        result=viginere_get_table_encrypt(table=vigin_table,
                                          keyword_letter="Y",
                                          text_letter="S")
        
        expected="Q"
        
        self.assertEqual(result, expected)
        
    def test_playfair_grid_list(self):
        
        input_var="GLAMORGAN"
        
        result=playfair_grid_list(keyword=input_var)
        
        expected=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']
        
        self.assertEqual(result, expected)
        
    def test_playfair_deal_with_dups(self):
        
        input_var="MEETMEATTREFFORESTSTATION"
        
        result=playfair_deal_with_dups(input_var)
        
        expected=["M","E","X","E","T","M","E","A","T","X","T","R","E","F","X",
                  "F","O","R","E","S","T","S","T","A","T","I","O","N"]
                
        self.assertEqual(result, expected)
        
    def test_get_play_fair_row_col(self):
        
        input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']
        
        result=get_play_fair_row_col(playfair_grid=input_var,
                                     letter="S")
        
        expected=(3,2)
        
        self.assertEqual(result, expected)

    def test_playfair_put_into_pairs(self):

        input_var=["A","B","C"]    

        result=playfair_put_into_pairs(text=input_var)

        expected=["AB","CX"]

        self.assertEqual(result, expected)

    def test_decrypt_pair(self):

      input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']

      result_1=playfair_decrypt_pair(playfair_list=input_var, 
                        pair="GP")

      result_2=playfair_decrypt_pair(playfair_list=input_var, 
                        pair="FI")

                        
      result_3=playfair_decrypt_pair(playfair_list=input_var, 
                        pair="MF")

      expected_1=("V","E")
      expected_2=("E","H")
      expected_3=("L","I")

      self.assertEqual(result_1, expected_1)
      self.assertEqual(result_2, expected_2)
      self.assertEqual(result_3, expected_3)

    def test_playfair_decrpt_row_pair(self):     

      
      input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']

      result_1=playfair_decrypt_row_pair(grid=input_var, 
                     pair="QT")
            
      result_2=playfair_decrypt_row_pair(grid=input_var, 
                     pair="EK")

      result_3=playfair_decrypt_row_pair(grid=input_var, 
                     pair="GL")

      result_4=playfair_decrypt_row_pair(grid=input_var, 
                     pair="VZ")


      expected_1=("P","S")
      expected_2=("K","I")
      expected_3=("O","G")
      expected_4=("Z","Y")
            
      self.assertEqual(result_1, expected_1)
      self.assertEqual(result_2, expected_2)
      self.assertEqual(result_3, expected_3)

    def test_playfair_decrpt_col_pair(self):     

      
      input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']

      result_1=playfair_decrypt_col_pair(grid=input_var, 
                     pair="LN")
            
      result_2=playfair_decrypt_col_pair(grid=input_var, 
                     pair="TC")

      expected_1=("W","L")
      expected_2=("I","M")
            
      self.assertEqual(result_1, expected_1)
      self.assertEqual(result_2, expected_2)

    def test_playfair_decrypt_rectangle(self):

      input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']

      result_1=playfair_decrypt_rectangle(grid=input_var, 
                                          x_1=0,
                                          y_1=3,
                                          x_2=2,
                                          y_2=1)
            
      result_2=playfair_decrypt_rectangle(grid=input_var, 
                                          x_1=0,
                                          y_1=0,
                                          x_2=1,
                                          y_2=1)

      expected_1=("L","I")
      expected_2=("L","R")
            
      self.assertEqual(result_1, expected_1)
      self.assertEqual(result_2, expected_2)

    def test_unpack_list_of_tuples(self):

      input_var=[("A","B"),("C","D")]
      result=unpack_list_of_tuples(list_tuples=input_var)

      expected=["A","B","C","D"]

      self.assertEqual(result, expected)

    def test_playfair_encrypt_row_pair(self):

      input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']
          
      result_1=playfair_encrypt_row_pair(grid=input_var,
                                    pair="AM")

      result_2=playfair_encrypt_row_pair(grid=input_var,
                                    pair="GO")

      result_3=playfair_encrypt_row_pair(grid=input_var,
                                    pair="VZ")

      expected_1=("M","O")                    
      expected_2=("L","G")
      expected_3=("W","V")


      self.assertEqual(result_1, expected_1)
      self.assertEqual(result_2, expected_2)
      self.assertEqual(result_3, expected_3)

    def test_playfair_encrypt_col_pair(self):

      input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']
          
      result_1=playfair_encrypt_col_pair(grid=input_var,
                                    pair="NF")

      result_2=playfair_encrypt_col_pair(grid=input_var,
                                    pair="MY")

      result_3=playfair_encrypt_col_pair(grid=input_var,
                                    pair="EP")

      expected_1=("F","Q")                    
      expected_2=("C","M")
      expected_3=("P","V")


      self.assertEqual(result_1, expected_1)
      self.assertEqual(result_2, expected_2)
      self.assertEqual(result_3, expected_3)

    def test_playfair_encrypt_rectangle(self):

      input_var=['G','L','A','M','O','R','N','B','C','D','E','F','H',
                  'I','K','P','Q','S','T','U','V','W','X','Y','Z']
          
      result_1=playfair_encrypt_rectangle(grid=input_var,
                                          x_1=2,
                                          y_1=0,
                                          x_2=0,
                                          y_2=2)

      result_2=playfair_encrypt_rectangle(grid=input_var,
                                          x_1=3,
                                          y_1=3,
                                          x_2=4,
                                          y_2=2)

      result_3=playfair_encrypt_rectangle(grid=input_var,
                                          x_1=3,
                                          y_1=3,
                                          x_2=1,
                                          y_2=0)
                                   
      expected_1=("H","G")                    
      expected_2=("S","Y")
      expected_3=("P","C")


      self.assertEqual(result_1, expected_1)
      self.assertEqual(result_2, expected_2)
      self.assertEqual(result_3, expected_3)

      



if __name__=="__main__":
    unittest.main()


        
        
    


import unittest

from crypto import (atbash_decrypt,
                    atbash_encrypt,
                    ceaser_decrypt,
                    ceaser_encrypt,
                    vigenere_decrypt,
                    vigenere_encrypt,
                    playfair_decrypt,
                    playfair_encrypt)


class test_crypto(unittest.TestCase):

    def test_atbash_decrypt(self):

        input_var_1="GSRHRHMLGEVIBHVXIVGZGZOO"
        input_var_2="gSRHRHMLGEViBHVXIVGZGZOo"
        input_var_3="GSR HRH MLG EVI BHV XIV GZG ZOO"


        result_1=atbash_decrypt(cipher_text=input_var_1)
        result_2=atbash_decrypt(cipher_text=input_var_2)
        result_3=atbash_decrypt(cipher_text=input_var_3)

        expected_1="THISISNOTVERYSECRETATALL"
        expected_2="THISISNOTVERYSECRETATALL"
        expected_3="THISISNOTVERYSECRETATALL"


        self.assertEqual(result_1,expected_1)
        self.assertEqual(result_2,expected_2)
        self.assertEqual(result_3,expected_3)
        self.assertRaises(TypeError, atbash_decrypt, 34)
        self.assertRaises(TypeError, atbash_decrypt, "exclamation!")

    def test_atbash_encrypt(self):
    
        input_var_1="THISISNOTVERYSECRETATALL"
        input_var_2="THISiSNOTVERYsECRETATALL"
        input_var_3="TH ISISN OTVER YSECRE TATALL"


        result_1=atbash_encrypt(plain_text=input_var_1)
        result_2=atbash_encrypt(plain_text=input_var_2)
        result_3=atbash_encrypt(plain_text=input_var_3)

        expected_1="GSRHRHMLGEVIBHVXIVGZGZOO"
        expected_2="GSRHRHMLGEVIBHVXIVGZGZOO"
        expected_3="GSRHRHMLGEVIBHVXIVGZGZOO"


        self.assertEqual(result_1,expected_1)
        self.assertEqual(result_2,expected_2)
        self.assertEqual(result_3,expected_3)
        self.assertRaises(TypeError, atbash_encrypt, 34)
        self.assertRaises(TypeError, atbash_encrypt, "exclamation!")

    def test_ceaser_decrypt(self):
    
        input_var_1="MTBVZNHPQDINIDTZGWJFPYMNX"
        input_var_2="MTBVZN HPQDINIDT ZGWJFPY MNX"
        input_var_3="MTbVZNHPQDInIDTZGWJFPYMNX"

        result_1=ceaser_decrypt(cipher_text=input_var_1,
                             shift=5)

        result_2=ceaser_decrypt(cipher_text=input_var_2,
                             shift=5)

        result_3=ceaser_decrypt(cipher_text=input_var_3,
                             shift=5)

        expected_1="HOWQUICKLYDIDYOUBREAKTHIS"
        expected_2="HOWQUICKLYDIDYOUBREAKTHIS"
        expected_3="HOWQUICKLYDIDYOUBREAKTHIS"

        self.assertEqual(result_1,expected_1)
        self.assertEqual(result_2,expected_2)
        self.assertEqual(result_3,expected_3)
        self.assertRaises(TypeError, ceaser_decrypt, 34,3)
        self.assertRaises(TypeError, ceaser_decrypt, "exclamation!",4)
        self.assertRaises(TypeError, ceaser_decrypt, "EXCLAMATION","A")
    
    def test_ceaser_encrypt(self):
        
        input_var_1="HOWQUICKLYDIDYOUBREAKTHIS"
        input_var_2="HOWQUICKLYDIDYOUBREAKTHis"
        input_var_3="HOW QUICKLY DID YOU BREAK THIS"

        result_1=ceaser_encrypt(plain_text=input_var_1,
                             shift=5)
        result_2=ceaser_encrypt(plain_text=input_var_2,
                             shift=5)

        result_3=ceaser_encrypt(plain_text=input_var_3,
                             shift=5)

        expected_1="MTBVZNHPQDINIDTZGWJFPYMNX"
        expected_2="MTBVZNHPQDINIDTZGWJFPYMNX"
        expected_3="MTBVZNHPQDINIDTZGWJFPYMNX"

        self.assertEqual(result_1,expected_1)
        self.assertEqual(result_2,expected_2)
        self.assertEqual(result_3,expected_3)

        self.assertRaises(TypeError, ceaser_encrypt, 34,3)
        self.assertRaises(TypeError, ceaser_encrypt, "exclamation!",4)
        self.assertRaises(TypeError, ceaser_encrypt, "EXCLAMATION","A")

    def test_vigenere_decrypt(self):
        
        input_var_1="FGQVEQYONMCCHAXTPB"
        input_var_2="F G Q V E Q Y O N M C C H A X T P B"
        input_var_3="YDXGJHCJVODXUGGZ"
        perm_3="thislepgywnomarkdbfcjquvxz"

        result_1a=vigenere_decrypt(cipher_text=input_var_1,
                                keyword="FAMILY",
                                permutation="")

        result_1b=vigenere_decrypt(cipher_text=input_var_1,
                                keyword="family",
                                permutation="")

        result_2a=vigenere_decrypt(cipher_text=input_var_2,
                                keyword="FAMILY",
                                permutation="")

        result_2b=vigenere_decrypt(cipher_text=input_var_2,
                                keyword="fam ily",
                                permutation="")

        result_3=vigenere_decrypt(cipher_text=input_var_3,
                                keyword="NIGHT time",
                                permutation=perm_3)


        expected_1="AGENTSTOBERECALLED"
        expected_2="AGENTSTOBERECALLED"
        expected_3="VISABILITYISPOOR"

        self.assertEqual(result_1a,expected_1)
        self.assertEqual(result_1b,expected_1)
        self.assertEqual(result_2a,expected_2)
        self.assertEqual(result_2b,expected_2)
        self.assertEqual(result_3,expected_3)

        self.assertRaises(TypeError, vigenere_decrypt, 34,3,5)
        self.assertRaises(TypeError, vigenere_decrypt, "exclamation!",4,3)
        self.assertRaises(TypeError, vigenere_decrypt, "EXCLAMATION","A","BB")


    def test_vigenere_encrypt(self):
        
        input_var_1="AGENTSTOBERECALLED"
        input_var_2="AGENTS TO BE RECALLED"
        input_var_3="VISABILITYISPOOR"
        perm_3="thislepgywnomarkdbfcjquvxz"

        result_1a=vigenere_encrypt(plain_text=input_var_1,
                                keyword="FAMILY",
                                permutation="")

        result_1b=vigenere_encrypt(plain_text=input_var_1,
                                keyword="family",
                                permutation="")

        result_2a=vigenere_encrypt(plain_text=input_var_2,
                                keyword="FAMILY",
                                permutation="")

        result_2b=vigenere_encrypt(plain_text=input_var_2,
                                keyword="fam ily",
                                permutation="")

        result_3=vigenere_encrypt(plain_text=input_var_3,
                                keyword="NIGHT time",
                                permutation=perm_3)


        expected_1="FGQVEQYONMCCHAXTPB"
        expected_2="FGQVEQYONMCCHAXTPB"
        expected_3="YDXGJHCJVODXUGGZ"

        self.assertEqual(result_1a,expected_1)
        self.assertEqual(result_1b,expected_1)
        self.assertEqual(result_2a,expected_2)
        self.assertEqual(result_2b,expected_2)
        self.assertEqual(result_3,expected_3)

        self.assertRaises(TypeError, vigenere_encrypt, 34,3,5)
        self.assertRaises(TypeError, vigenere_encrypt, "exclamation!",4,3)
        self.assertRaises(TypeError, vigenere_encrypt, "EXCLAMATION","A","BB")


    def test_playfair_decrypt(self):
        
        input_var_1="GIVHYCHGSYPCFHWHGDHPUTSMYTLD"
        input_var_2="GI VH YC HG SY PC FH WH GD HP UT SM YT LD"


        result_1a=playfair_decrypt(cipher_text=input_var_1,
                                keyword="GLAMORGAN")

        result_1b=playfair_decrypt(cipher_text=input_var_1,
                                keyword="GLAm ORGAN")

        result_2=playfair_decrypt(cipher_text=input_var_2,
                                keyword="GLAMORGAN")

        expected="MEETMEATTREFFORESTSTATION"

        self.assertEqual(result_1a,expected)
        self.assertEqual(result_1b,expected)
        self.assertEqual(result_2,expected)

        self.assertRaises(TypeError, playfair_decrypt, 34,3)
        self.assertRaises(TypeError, playfair_decrypt, "exclamation!","A")
        self.assertRaises(TypeError, playfair_decrypt, "EXCLAMATION",6)


    def test_playfair_encrypt(self):
        
        input_var_1="MEETMEATTREFFORESTSTATION"
        input_var_2="MEET ME at TREFFOREST STATION"


        result_1a=playfair_encrypt(plain_text=input_var_1,
                                keyword="GLAMORGAN")

        result_1b=playfair_encrypt(plain_text=input_var_1,
                                keyword="GLAm ORGAN")

        result_2=playfair_encrypt(plain_text=input_var_2,
                                keyword="GLAMORGAN")

        expected="GIVHYCHGSYPCFHWHGDHPUTSMYTLD"

        self.assertEqual(result_1a,expected)
        self.assertEqual(result_1b,expected)
        self.assertEqual(result_2,expected)

        self.assertRaises(TypeError, playfair_encrypt, 34,3)
        self.assertRaises(TypeError, playfair_encrypt, "exclamation!","A")
        self.assertRaises(TypeError, playfair_encrypt, "EXCLAMATION",6)

if __name__=="__main__":
        unittest.main()





















from setuptools import setup

with open("README.md","r") as fh:
      long_description=fh.read()

setup(name='cryptology',
      version='0.0.1',
      description='Decrypt/Encrypt text using various cipher techniques',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='Antonio Felton',
      author_email='',
      py_modules=['crypto'],
      package_dir={'':'cryptology'}
)